@@ -13,7 +13,7 @@ config = Config(
     executors=[
         HighThroughputExecutor(
             label='OSG_HTEX',
-            max_workers=1,
+            max_workers_per_node=1,
             encrypted=True,
             provider=CondorProvider(
                 nodes_per_block=1,
